# prevent defaulting to the help rule
all:

ifneq ($(ARM_MAKEFILE_FLAGS_SET),1) # include guard
ifndef PE_ENV # compiler detection not needed on crays

ifneq ($(filter default undefined,$(origin FC)),)
# default to GNU
export F77 ?= gfortran
export F90 ?= gfortran
export FC  := gfortran
else
# prefer FC if defined
export F77 ?= $(FC)
export F90 ?= $(FC)
endif

# MPI C/C++ Compilers
ifndef MPICC
ifeq ($(shell which mpiicc > /dev/null 2>&1; echo $$?),0)
export MPICC := mpiicc
else ifeq ($(shell which mpicc > /dev/null 2>&1; echo $$?),0)
export MPICC := mpicc
else
$(warning Could not detect MPI C compiler)
endif
endif

ifndef MPICXX
ifeq ($(shell which mpiicpc > /dev/null 2>&1; echo $$?),0)
export MPICXX := mpiicpc
else ifeq ($(shell which mpicxx > /dev/null 2>&1; echo $$?),0)
export MPICXX := mpicxx
else
$(warning Could not detect MPI C++ compiler)
endif
endif

MPICC_VERSION := $(shell $(MPICC) --version 2>&1)

# MPI Fortran Compilers
ifndef MPIF90
ifeq ($(shell which mpiifort > /dev/null 2>&1; echo $$?),0)
export MPIF90 := mpiifort
else ifeq ($(shell which mpifc > /dev/null 2>&1; echo $$?),0)
export MPIF90 := mpifc
else ifeq ($(shell which mpifort > /dev/null 2>&1; echo $$?),0)
export MPIF90 := mpifort
else ifeq ($(shell which mpif90 > /dev/null 2>&1; echo $$?),0)
export MPIF90 := mpif90
else
$(warning Could not detect MPI Fortran compiler)
endif
endif

ifndef MPIF77
ifeq ($(shell which mpif77 > /dev/null 2>&1; echo $$?),0)
export MPIF77 := mpif77
else
export MPIF77 := $(MPIF90)
endif
endif

MPIF90_VERSION := $(shell $(MPIF90) --version 2>&1)
ifneq ($(shell echo $$?),0)
# For IBM XLF
MPIF90_VERSION := $(shell $(MPIF90) -qversion 2>&1)
endif

endif # PE_ENV
export MPIFC ?= $(MPIF90)

.PHONY: help
help:
	@echo -e " --------------------- Arm HPC examples ---------------------\n\n" \
                 "Use 'make' to build the example(s)\n" \
                 "(compiler invocations will be detected from the environment)\n\n" \
                 "Detecting from environment:\n" \
                 "CC=$(CC) CXX=$(CXX)\n" \
                 "F77=$(F77) F90=$(F90) FC=$(FC)\n" \
                 "MPICC=$(MPICC) MPICXX=$(MPICXX)\n" \
                 "MPIF77=$(MPIF77) MPIF90=$(MPIF90) MPIFC=$(MPIFC)\n\n" \
                 "Redefine the above variables in the environment if necessary\n" \
                 "------------------------------------------------------------"

# Flags for compiler inlining: MAP works whether inlining is on or off,
# but you'll typically see more intuitive stacks with it turned off.
# The major compilers are discussed here:
#
# Intel: -g -fno-inline -no-ip -no-ipo -fno-omit-frame-pointer -O3 is
# recommended. At O3 the compiler doesn't produce enough unwind info even
# with -debug inline-debug-info set.
#
# PGI: -g -O3 -Meh_frame is recommended. Other settings dont produce enough
# unwind information for inlined functions otherwise. This adds some
# performance penalty - around 8% is typical.
#
# GNU: -g -O3 -fno-inline is recommended. You might be lucky without -fno-inline,
# as it should produce enough information to unwind those calls. You will see
# my_function [inlined] in the MAP stack for functions that were inline.
# -fno-inline-functions appears with newer gnu compilers, just to confuse

# Common OpenMP flags for supported compilers
# -fopenmp for gnu
# -openmp  for intel
# -mp      for pgi
# -qsmp=omp:noopt for IBM
# -homp    for cray (compiler)

# Common pthread flags for supported compilers
# -pthread for GNU
# -lpthread for other compilers

# Serial C/C++ Compilers
LOCAL_PTHREAD_CFLAG := -lpthread
ifneq ($(or $(findstring icc,$(CC)),$(findstring INTEL,$(PE_ENV))),)
	LOCAL_MAP_CFLAGS := -g -fno-inline -no-ip -no-ipo -fno-omit-frame-pointer -O3
	LOCAL_OPENMP_CFLAG := -qopenmp
else ifneq ($(or $(findstring pgcc,$(CC)),$(findstring PGI,$(PE_ENV))),)
	LOCAL_MAP_CFLAGS := -g -Meh_frame -Mframe -O3
	LOCAL_OPENMP_CFLAG := -mp
else ifneq ($(or $(findstring xlc,$(CC)),$(findstring IBM,$(PE_ENV))),)
	LOCAL_MAP_CFLAGS := -g -O3 -qnoinline
	LOCAL_OPENMP_CFLAGS := -qsmp=omp:noopt
else ifeq ($(PE_ENV),CRAY)
	LOCAL_MAP_CFLAGS := -g -O3 -hipa0
	LOCAL_OPENMP_CFLAG := -homp
else # Default to GNU
	LOCAL_MAP_CFLAGS := -g -O3 -fno-inline
	LOCAL_OPENMP_CFLAG := -fopenmp
	LOCAL_PTHREAD_CFLAG := -pthread
endif

# MPI C/C++ Compilers
LOCAL_MPI_PTHREAD_CFLAG := -lpthread
ifneq ($(or $(findstring icc,$(MPICC_VERSION)),$(findstring INTEL,$(PE_ENV))),)
	LOCAL_MPI_MAP_CFLAGS := -g -fno-inline -no-ip -no-ipo -fno-omit-frame-pointer -O3
	LOCAL_MPI_OPENMP_CFLAG := -qopenmp
else ifneq ($(or $(findstring pgcc,$(MPICC_VERSION)),$(findstring PGI,$(PE_ENV))),)
	LOCAL_MPI_MAP_CFLAGS := -g -Meh_frame -Mframe -O3
	LOCAL_MPI_OPENMP_CFLAG := -mp
else ifneq ($(or $(findstring xlc,$(MPICC_VERSION)),$(findstring IBM,$(PE_ENV))),)
	LOCAL_MPI_MAP_CFLAGS := -g -O3 -qnoinline
	LOCAL_MPI_OPENMP_CFLAG := -qsmp=omp:noopt
else ifeq ($(PE_ENV),CRAY)
	LOCAL_MPI_MAP_CFLAGS := -g -O3 -hipa0
	LOCAL_MPI_OPENMP_CFLAG := -homp
else # Default to GNU
	LOCAL_MPI_MAP_CFLAGS := -g -O3 -fno-inline
	LOCAL_MPI_OPENMP_CFLAG := -fopenmp
	LOCAL_MPI_PTHREAD_CFLAG := -pthread
endif

# Serial Fortran Compilers
LOCAL_PTHREAD_FCFLAG := -lpthread
ifneq ($(or $(findstring ifort,$(FC)),$(findstring INTEL,$(PE_ENV))),)
	LOCAL_MAP_FCFLAGS := -g -fno-inline -no-ip -no-ipo -fno-omit-frame-pointer -O3
	LOCAL_OPENMP_FCFLAG := -qopenmp
else ifneq ($(or $(findstring pgfortran,$(FC)),$(findstring PGI,$(PE_ENV))),)
	LOCAL_MAP_FCFLAGS := -g -Meh_frame -Mframe -O3
	LOCAL_OPENMP_FCFLAG := -mp
else ifneq ($(or $(findstring xlf,$(FC)),$(findstring IBM,$(PE_ENV))),)
	LOCAL_MAP_FCFLAGS := -g -O3 -qnoinline
	LOCAL_OPENMP_FCFLAG := -qsmp=omp:noopt -qnohot -lxlf90 -lxlsmp -lxlfmath
else ifeq ($(PE_ENV),CRAY)
	LOCAL_MAP_FCFLAGS := -g -O3 -hipa0
	LOCAL_OPENMP_FCFLAG := -homp
else # Default to GNU
	LOCAL_MAP_FCFLAGS := -g -O3 -fno-inline
	LOCAL_OPENMP_FCFLAG := -fopenmp
	LOCAL_PTHREAD_FCFLAG := -pthread
endif

# MPI Fortran Compilers
LOCAL_MPI_PTHREAD_FCFLAG := -lpthread
ifneq ($(or $(findstring IFORT,$(MPIF90_VERSION)),$(findstring INTEL,$(PE_ENV))),)
	LOCAL_MPI_MAP_FCFLAGS := -g -fno-inline -no-ip -no-ipo -fno-omit-frame-pointer -O3
	LOCAL_MPI_OPENMP_FCFLAG := -qopenmp
else ifneq ($(or $(findstring pgfortran,$(MPIF90_VERSION)),$(findstring PGI,$(PE_ENV))),)
	LOCAL_MPI_MAP_FCFLAGS := -g -Meh_frame -Mframe -O3
	LOCAL_MPI_OPENMP_FCFLAG := -mp
else ifneq ($(or $(findstring xlf,$(MPIF90_VERSION)),$(findstring IBM,$(PE_ENV))),)
	LOCAL_MPI_MAP_FCFLAGS := -g -O3 -qnoinline
	LOCAL_MPI_OPENMP_FCFLAG := -qsmp=omp:noopt -qnohot -lxlf90 -lxlsmp -lxlfmath
else ifeq ($(PE_ENV),CRAY)
	LOCAL_MPI_MAP_FCFLAGS := -g -O3 -hipa0
	LOCAL_MPI_OPENMP_FCFLAG := -homp
else # Default to GNU
	LOCAL_MPI_MAP_FCFLAGS := -g -O3 -fno-inline
	LOCAL_MPI_OPENMP_FCFLAG := -fopenmp
	LOCAL_MPI_PTHREAD_FCFLAG := -pthread
endif

# Allow overrides
export MAP_FCFLAGS ?= $(LOCAL_MAP_FCFLAGS)
export MAP_CFLAGS ?= $(LOCAL_MAP_CFLAGS)
export OPENMP_CFLAG ?= $(LOCAL_OPENMP_CFLAG)
export OPENMP_FCFLAG ?= $(LOCAL_OPENMP_FCFLAG)
export PTHREAD_CFLAG ?= $(LOCAL_PTHREAD_CFLAG)
export PTHREAD_FCFLAG ?= $(LOCAL_PTHREAD_FCFLAG)
export MPI_MAP_FCFLAGS ?= $(LOCAL_MPI_MAP_FCFLAGS)
export MPI_MAP_CFLAGS ?= $(LOCAL_MPI_MAP_CFLAGS)
export MPI_OPENMP_CFLAG ?= $(LOCAL_MPI_OPENMP_CFLAG)
export MPI_OPENMP_FCFLAG ?= $(LOCAL_MPI_OPENMP_FCFLAG)
export MPI_PTHREAD_CFLAG ?= $(LOCAL_MPI_PTHREAD_CFLAG)
export MPI_PTHREAD_FCFLAG ?= $(LOCAL_MPI_PTHREAD_FCFLAG)

# Call the Cray compiler wrappers
ifdef PE_ENV
export CC = cc
export CXX = CC
export MPICC = cc
export MPICXX = CC
export F77 = ftn
export F90 = ftn
export FC = ftn
export MPIF77 = ftn
export MPIF90 = ftn
endif

export ARM_MAKEFILE_FLAGS_SET := 1 # include guard
endif # ARM_MAKEFILE_FLAGS_SET
